import java.io.*;
import java.net.*;
import java.nio.*;
import java.nio.channels.*;
import java.nio.charset.StandardCharsets;

public class MyHttpClientNio {
    private static final long POLL_TIMEOUT = 5000L; // select\bh̃^CAEgli~bj
    private enum HttpState {
        SEND_REQUEST,
        RECV_RESPONSE,
    };

    private final String serverHost;
    private final int serverPort;

    public MyHttpClientNio(String serverHost, int serverPort) {
        this.serverHost = serverHost;
        this.serverPort = serverPort;
    }

    public void execHttpClient() {
        try (Selector selector = Selector.open();
             SocketChannel channel = SocketChannel.open()) {  // \I\Pbg
            connect(channel);
            channel.configureBlocking(false); // mubLO[h

            HttpState httpState = HttpState.SEND_REQUEST;

            loop:
            while (true) {
                switch (httpState) {
                case SEND_REQUEST:
                    channel.register(selector, SelectionKey.OP_WRITE);
                    break;
                case RECV_RESPONSE:
                    channel.register(selector, SelectionKey.OP_READ);
                    break;
                default:
                    assert(false);
                }

                if (selector.select(POLL_TIMEOUT) > 0) {
                    for (SelectionKey key : selector.selectedKeys()) {
                        if (httpState == HttpState.SEND_REQUEST && key.isWritable()) {
                            sendRequest(channel);
                            httpState = HttpState.RECV_RESPONSE;
                        } else if (httpState == HttpState.RECV_RESPONSE && key.isReadable()) {
                            recvResponse(channel);
                            break loop;
                        } else {
                            assert(false);
                        }
                    }
                    selector.selectedKeys().clear();
                } else {
                    if (askContinue()) {
                        httpState = HttpState.SEND_REQUEST;
                        continue;
                    } else {
                        break loop;
                    }
                }
            }
        } catch (IOException e) {
            throw new UncheckedIOException(e); // sOɗO|ijava.io.UncheckedIOExceptionJava8ȍ~̃NXj
        }
    }

    // \I\Pbg̐ڑ
    private void connect(SocketChannel channel) throws IOException {
        InetSocketAddress address = new InetSocketAddress(this.serverHost, this.serverPort);
        channel.connect(address);
    }

    // \Pbg̃f[^M
    private void sendRequest(SocketChannel channel) throws IOException {
        ByteBuffer buffer = ByteBuffer.allocate(4096); // TCYȂꍇ̏͏ȗ
        buffer.put("GET / HTTP/1.1\r\n".getBytes());
        buffer.put("Host: localhost\r\n".getBytes());
        buffer.put("\r\n".getBytes());
        buffer.flip();

        channel.write(buffer); // ݃oCg̃`FbN͏ȗ
    }

    // \Pbg̃f[^M
    private void recvResponse(SocketChannel channel) throws IOException {
        ByteBuffer buffer = ByteBuffer.allocate(4096); // TCYȂꍇ̏͏ȗ
        channel.read(buffer); // ǂݍ݃oCg̃`FbN͏ȗ
        System.out.println(StandardCharsets.UTF_8.decode(buffer));
    }

    private boolean askContinue() {
        System.out.println("continue? [y/n]");
        String input = System.console().readLine();
        return input.charAt(0) == 'y';
    }

    public static void main(String... args) {
        if (args.length < 1) {
            System.err.println("usage: java " + MyHttpClientNio.class.getSimpleName() + " WebT[ozXg");
            System.exit(-1);
        }

        MyHttpClientNio my = new MyHttpClientNio(args[0], 80);
        my.execHttpClient();
    }
}
