import javafx.beans.property.IntegerProperty;
import javafx.beans.property.ReadOnlyLongProperty;
import javafx.beans.property.ReadOnlyLongPropertyBase;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;

class MyBean {
    private IntegerProperty width;
    private IntegerProperty height;
    private SizeProperty size;
    
    public int getWidth() {
        return width == null ? 0 : width.get();
    }

    public void setWidth(int value) {
        widthProperty().set(value);
    }

    public IntegerProperty widthProperty() {
        if (width == null) {
            width = new SimpleIntegerProperty(this, "width");
        }
        return width;
    }

    public int getHeight() {
        return height == null ? 0 : heightProperty().get();
    }

    public void setHeight(int value) {
        heightProperty().set(value);
    }
    
    public IntegerProperty heightProperty() {
        if (height == null) {
            height = new SimpleIntegerProperty(this, "height");
        }
        return height;
    }
    
    public long getSize() { return size == null ? 0 : sizeProperty().get(); }
    
    class SizeProperty extends ReadOnlyLongPropertyBase 
        implements ChangeListener {
        private boolean valid;
        private long value;
        
        public SizeProperty() {
            widthProperty().addListener(this);
            heightProperty().addListener(this);
        }

        @Override
        public long get() {
            if (!valid) {
                value = getWidth() * getHeight();
                valid = true;
            }
            return value;
        }

        @Override
        public Object getBean() {
            return MyBean.this;
        }

        @Override
        public String getName() {
            return "size";
        }

        @Override
        public void changed(ObservableValue observable, Object oldValue, Object newValue) {
            valid = false;
            fireValueChangedEvent();
        }
    }
    
    public ReadOnlyLongProperty sizeProperty() {
        if (size == null) {
            size = new SizeProperty();
        }
        return size;
    }
}
