import javafx.application.Application;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ProgressBar;
import javafx.scene.layout.HBox;
import javafx.stage.Stage;

public class MyApplication extends Application {
    @Override
    public void start(Stage primaryStage) throws Exception {
        ProgressBar progressBar = new ProgressBar(0.0);
        Button runButton = new Button("Run");
        Button cancelButton = new Button("Cancel");
        cancelButton.setDisable(true);

        class MyTask implements Runnable {
            private boolean cancelled;
            public void cancel() {
                cancelled = true;
            }
            public boolean isCancelled() {
                return cancelled;
            }

            @Override
            public void run() {
                for (int i = 0; i <= 100; i++) {
                    if (isCancelled()) {
                        break;
                    }
                    final double progress = i/100d;
                    Platform.runLater(new Runnable() {
                        @Override
                        public void run() {
                            progressBar.setProgress(progress);
                        }
                    });
                    try {
                        Thread.sleep(10);
                    } catch (InterruptedException e) {
                    }
                }
                Platform.runLater(new Runnable() {
                    @Override
                    public void run() {
                        runButton.setDisable(false);
                        cancelButton.setDisable(true);
                    }
                });
            }
        }
        final MyTask[] currentTask = new MyTask[1];
        
        runButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent event) {
                runButton.setDisable(true);
                cancelButton.setDisable(false);
                
                MyTask task = new MyTask();
                currentTask[0] = task;
                
                Thread thread = new Thread(task);
                thread.setDaemon(true);
                thread.start();
            }
        });
        cancelButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent event) {
                currentTask[0].cancel();            
            }
        });
        
        HBox root = new HBox(runButton, cancelButton, progressBar);
        root.setAlignment(Pos.CENTER);
        root.setSpacing(10);
        
        primaryStage.setScene(new Scene(root, 300, 200));
        primaryStage.setTitle("Background Task");
        primaryStage.show();
    }
}
