import javafx.application.Application;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ProgressBar;
import javafx.scene.layout.HBox;
import javafx.stage.Stage;

import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

public class MyApplication extends Application {
    @Override
    public void start(Stage primaryStage) throws Exception {
        ProgressBar progressBar = new ProgressBar(0.0);
        Button runButton = new Button("Run");
        Button cancelButton = new Button("Cancel");
        cancelButton.setDisable(true);
        
        class MyTask extends Task<Void> {
            @Override
            public Void call() throws Exception {
                for (int i = 0; i <= 100; i++) {
                    if (isCancelled()) {
                        break;
                    }
                    updateProgress(i, 100);
                    try {
                        Thread.sleep(10);
                    } catch (InterruptedException e) {
                    }
                }
                return null;
            }
        }
        
        class MyService extends Service<Void> {
            public MyService() {
                setExecutor(Executors.newCachedThreadPool(new ThreadFactory() {
                    @Override
                    public Thread newThread(Runnable r) {
                        Thread t = new Thread(r);                      
                        t.setDaemon(true);
                        return t;
                    }
                }));
            }
            @Override
            protected MyTask createTask() {
                return new MyTask();
            }
        }
        
        MyService service = new MyService();
        service.runningProperty().addListener(new ChangeListener<Boolean>() {
            @Override
            public void changed(ObservableValue<? extends Boolean> observable,
                                Boolean oldValue, Boolean newValue) {
                if (!newValue) {
                    cancelButton.setDisable(true);
                    runButton.setDisable(false);
                }
            }
        });
        service.progressProperty().addListener(new ChangeListener<Number>() {
            @Override
            public void changed(ObservableValue<? extends Number> observable, Number oldValue, Number newValue) {
                progressBar.setProgress(newValue.doubleValue());
            }
        });
        
        runButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent event) {
                runButton.setDisable(true); 
                cancelButton.setDisable(false);
                
                service.reset(); 
                service.start();
            }
        });
        cancelButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent event) {
                service.cancel();
            }
        });
        
        HBox root = new HBox(runButton, cancelButton, progressBar);
        root.setAlignment(Pos.CENTER);
        root.setSpacing(10);
        
        primaryStage.setScene(new Scene(root, 300, 200));
        primaryStage.setTitle("Background Task");
        primaryStage.show();
    }
}
