import javafx.beans.binding.Bindings;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.ReadOnlyDoubleWrapper;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.css.*;
import javafx.scene.control.Control;
import javafx.scene.control.Skin;
import javafx.scene.paint.Color;
import javafx.scene.paint.Paint;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

public class Knob extends Control {
    private DoubleProperty minValue = new SimpleDoubleProperty(this, "minValue", 0);
    public DoubleProperty minValueProperty() {
        return minValue;
    }
    public void setMinValue(double value) {
        minValueProperty().set(value);
    }
    public double getMinValue() {
        return minValueProperty().get();
    }
    
    private DoubleProperty maxValue = new SimpleDoubleProperty(this, "maxValue", 100);
    public DoubleProperty maxValueProperty() {
        return maxValue; 
    }
    public void setMaxValue(double value) {
        maxValueProperty().set(value);
    }
    public double getMaxValue() {
        return maxValueProperty().get();
    }

    private DoubleProperty value = new SimpleDoubleProperty(this, "value", 0);
    public DoubleProperty valueProperty() {
        return value;
    }
    public void setValue(double value) {
        valueProperty().set(value);
    }
    public double getValue() {
        return valueProperty().get();
    }

    private DoubleProperty totalAngle = new SimpleStyleableDoubleProperty(
            TOTAL_ANGLE, this, "totalAngle", 270d);
    public DoubleProperty totalAngleProperty() {
        return totalAngle;
    }
    public void setTotalAngle(double value) {
        totalAngleProperty().set(value);
    }
    public double getTotalAngle() {
        return totalAngleProperty().get();
    }
    
    private DoubleProperty startAngle = new ReadOnlyDoubleWrapper(this, "startAngle");
    {
        startAngle.bind(Bindings.subtract(360, totalAngleProperty()).divide(2).add(90));
    }
    public double getStartAngle() {
        return startAngle.get();
    }

    private DoubleProperty bodyRadius = new SimpleStyleableDoubleProperty(
            BODY_RADIUS, this, "bodyRadius", 30d);
    public DoubleProperty bodyRadiusProperty() {
        return bodyRadius;
    }
    public void setBodyRadius(double value) {
        bodyRadiusProperty().set(value);
    }
    public double getBodyRadius() {
        return bodyRadiusProperty().get();
    }
    
    private ObjectProperty<Paint> bodyColor = new SimpleStyleableObjectProperty<Paint>(
            BODY_COLOR, this, "bodyColor", Color.LIGHTGRAY);
    public ObjectProperty<Paint> bodyColorProperty() {
        return bodyColor;
    }
    public void setBodyColor(Paint value) {
        bodyColorProperty().set(value);
    }
    public Paint getBodyColor() {
        return bodyColorProperty().get();
    }
    
    private DoubleProperty markerRadius = new SimpleStyleableDoubleProperty(
            MARKER_RADIUS, this, "markerRadius", 3d);
    public DoubleProperty markerRadiusProperty() {
        return markerRadius;
    }
    public void setMarkerRadius(double value) {
        markerRadiusProperty().set(value);
    }
    public double getMarkerRadius() {
        return markerRadiusProperty().get();
    }

    private DoubleProperty markerSpacing = new SimpleStyleableDoubleProperty(
            MARKER_SPACING, this, "markerSpacing", 4d);
    public DoubleProperty markerSpacingProperty() {
        return markerSpacing;
    }
    public void setMarkerSpacing(double value) {
        markerSpacingProperty().set(value);
    }
    public double getMarkerSpacing() {
        return markerSpacingProperty().get();
    }
    
    private ObjectProperty<Paint> markerColor = new SimpleStyleableObjectProperty<Paint>(
            MARKER_COLOR, this, "markerColor", Color.GRAY);
    public ObjectProperty<Paint> markerColorProperty() {
        return markerColor;
    }
    public void setMarkerColor(Paint value) {
        markerColorProperty().set(value);
    }
    public Paint getMarkerColor() {
        return markerColorProperty().get();
    }

    private DoubleProperty tickRadius = new SimpleStyleableDoubleProperty(
            TICK_RADIUS, this, "tickRadius", 2d);
    public DoubleProperty tickRadiusProperty() {
        return tickRadius;
    }
    public void setTickRadius(double value) {
        tickRadiusProperty().set(value);
    }
    public double getTickRadius() {
        return tickRadiusProperty().get();
    }

    private DoubleProperty tickSpacing = new SimpleStyleableDoubleProperty(
            TICK_SPACING, this, "tickSpacing", 10d);
    public DoubleProperty tickSpacingProperty() {
        return tickSpacing;
    }
    public void setTickSpacing(double value) {
        tickSpacingProperty().set(value);
    }
    public double getTickSpacing() {
        return tickSpacingProperty().get();
    }

    private ObjectProperty<Paint> tickColor = new SimpleStyleableObjectProperty<Paint>(
            TICK_COLOR, this, "tickColor", Color.DARKGRAY);
    public ObjectProperty<Paint> tickColorProperty() {
        return tickColor;
    }
    public void setTickColor(Paint value) {
        tickColorProperty().set(value);
    }
    public Paint getTickColor() {
        return tickColorProperty().get();
    }

    private ObjectProperty<Paint> highlightColor = new SimpleStyleableObjectProperty<Paint>(
            SHADOW_COLOR, this, "highlightColor", Color.WHITE);
    public ObjectProperty<Paint> highlightColorProperty() {
        return highlightColor;
    }
    public void setHighlightColor(Paint value) {
        highlightColorProperty().set(value);
    }
    public Paint getHighlightColor() {
        return highlightColorProperty().get();
    }

    private ObjectProperty<Paint> shadowColor = new SimpleStyleableObjectProperty<Paint>(
            SHADOW_COLOR, this, "shadowColor", Color.GRAY);
    public ObjectProperty<Paint> shadowColorProperty() {
        return shadowColor;
    }
    public void setShadowColor(Paint value) {
        shadowColorProperty().set(value);
    }
    public Paint getShadowColor() {
        return shadowColorProperty().get();
    }

    private DoubleProperty borderWidth = new SimpleStyleableDoubleProperty(
            BORDER_WIDTH, this, "borderWidth", 3d);
    public DoubleProperty borderWidthProperty() {
        return borderWidth;
    }
    public void setBorderWidth(double value) {
        borderWidthProperty().set(value);
    }
    public double getBorderWidth() {
        return borderWidthProperty().get();
    }

    private ObjectProperty<Paint> borderColor = new SimpleStyleableObjectProperty<Paint>(
            BORDER_COLOR, this, "borderColor", Color.GRAY);
    public ObjectProperty<Paint> borderColorProperty() {
        return borderColor;
    }
    public void setBorderColor(Paint value) {
        borderColorProperty().set(value);
    }
    public Paint getBorderColor() {
        return borderColorProperty().get();
    }
    
    private DoubleProperty borderAngle = new SimpleStyleableDoubleProperty(
            BORDER_ANGLE, this, "borderAngle", 45d);
    public DoubleProperty borderAngleProperty() {
        return borderAngle;
    }
    public void setBorderAngle(double value) {
        borderAngleProperty().set(value);
    }
    public double getBorderAngle() {
        return borderAngleProperty().get();
    }
    
    private static CssMetaData<Knob, Number> TOTAL_ANGLE = new CssMetaData<Knob, Number>(
            "-my-total-angle", StyleConverter.getSizeConverter(), 270d) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.totalAngleProperty().isBound();
        }

        @Override
        public StyleableProperty<Number> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Number>) knob.totalAngleProperty();
        }
    };

    private static CssMetaData<Knob, Number> BODY_RADIUS = new CssMetaData<Knob, Number>(
            "-my-body-radius", StyleConverter.getSizeConverter(), 30d) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.bodyRadiusProperty().isBound();
        }

        @Override
        public StyleableProperty<Number> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Number>) knob.bodyRadiusProperty();
        }
    };
    private static CssMetaData<Knob, Paint> BODY_COLOR = new CssMetaData<Knob, Paint>(
            "-my-body-color", StyleConverter.getPaintConverter(), Color.LIGHTGRAY) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.bodyColorProperty().isBound();
        }

        @Override
        public StyleableProperty<Paint> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Paint>) knob.bodyColorProperty();
        }
    };
    private static CssMetaData<Knob, Number> MARKER_RADIUS = new CssMetaData<Knob, Number>(
            "-my-marker-radius", StyleConverter.getSizeConverter(), 3d) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.markerRadiusProperty().isBound();
        }

        @Override
        public StyleableProperty<Number> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Number>) knob.markerRadiusProperty();
        }
    };
    private static CssMetaData<Knob, Number> MARKER_SPACING = new CssMetaData<Knob, Number>(
            "-my-marker-spacing", StyleConverter.getSizeConverter(), 4d) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.markerSpacingProperty().isBound();
        }

        @Override
        public StyleableProperty<Number> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Number>) knob.markerSpacingProperty();
        }
    };
    private static CssMetaData<Knob, Paint> MARKER_COLOR = new CssMetaData<Knob, Paint>(
            "-my-marker-color", StyleConverter.getPaintConverter(), Color.TRANSPARENT) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.markerColorProperty().isBound();
        }

        @Override
        public StyleableProperty<Paint> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Paint>) knob.markerColorProperty();
        }
    };
    private static CssMetaData<Knob, Number> TICK_RADIUS = new CssMetaData<Knob, Number>(
            "-my-tick-radius", StyleConverter.getSizeConverter(), 3d) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.tickRadiusProperty().isBound();
        }

        @Override
        public StyleableProperty<Number> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Number>) knob.tickRadiusProperty();
        }
    };
    private static CssMetaData<Knob, Number> TICK_SPACING = new CssMetaData<Knob, Number>(
            "-my-tick-spacing", StyleConverter.getSizeConverter(), 10d) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.tickSpacingProperty().isBound();
        }

        @Override
        public StyleableProperty<Number> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Number>) knob.tickSpacingProperty();
        }
    };
    private static CssMetaData<Knob, Paint> TICK_COLOR = new CssMetaData<Knob, Paint>(
            "-my-tick-color", StyleConverter.getPaintConverter(), Color.DARKGRAY) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.tickColorProperty().isBound();
        }

        @Override
        public StyleableProperty<Paint> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Paint>) knob.tickColorProperty();
        }
    };
    private static CssMetaData<Knob, Paint> HIGHLIGHT_COLOR = new CssMetaData<Knob, Paint>(
            "-my-highlight-color", StyleConverter.getPaintConverter(), Color.DARKGRAY) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.highlightColorProperty().isBound();
        }

        @Override
        public StyleableProperty<Paint> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Paint>) knob.highlightColorProperty();
        }
    };
    private static CssMetaData<Knob, Paint> SHADOW_COLOR = new CssMetaData<Knob, Paint>(
            "-my-shadow-color", StyleConverter.getPaintConverter(), Color.GRAY) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.shadowColorProperty().isBound();
        }

        @Override
        public StyleableProperty<Paint> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Paint>) knob.shadowColorProperty();
        }
    };
    private static CssMetaData<Knob, Number> BORDER_WIDTH = new CssMetaData<Knob, Number>(
            "-my-border-width", StyleConverter.getSizeConverter(), 3d) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.borderWidthProperty().isBound();
        }

        @Override
        public StyleableProperty<Number> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Number>) knob.borderWidthProperty();
        }
    };
    private static CssMetaData<Knob, Paint> BORDER_COLOR = new CssMetaData<Knob, Paint>(
            "-my-border-color", StyleConverter.getPaintConverter(), Color.GRAY) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.borderColorProperty().isBound();
        }

        @Override
        public StyleableProperty<Paint> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Paint>) knob.borderColorProperty();
        }
    };
    private static CssMetaData<Knob, Number> BORDER_ANGLE = new CssMetaData<Knob, Number>(
            "-my-border-angle", StyleConverter.getSizeConverter(), 45) {
        @Override
        public boolean isSettable(Knob knob) {
            return !knob.borderAngleProperty().isBound();
        }

        @Override
        public StyleableProperty<Number> getStyleableProperty(Knob knob) {
            return (StyleableProperty<Number>) knob.borderAngleProperty();
        }
    };

    private static List<CssMetaData<? extends Styleable, ?>> STYLEABLES = 
            Collections.unmodifiableList(Arrays.asList(
                    BODY_RADIUS, BODY_COLOR, MARKER_RADIUS, MARKER_COLOR,
                    TICK_COLOR, HIGHLIGHT_COLOR, SHADOW_COLOR, BORDER_WIDTH,
                    BORDER_COLOR, BORDER_ANGLE));

    @Override
    protected List<CssMetaData<? extends Styleable, ?>> getControlCssMetaData() {
        return STYLEABLES;
    }

    private static final String DEFAULT_STYLE_CLASS = "knob";
    
    private void initialize() {
        getStyleClass().setAll(DEFAULT_STYLE_CLASS);
    }
        
    public Knob() {
        initialize();
    }

    public Knob(double value) {
        setValue(value);
        initialize();
    }

    @Override
    protected Skin<?> createDefaultSkin() {
        return new KnobSkin(this);
    }
}
