import com.sun.javafx.css.StyleManager;
import javafx.application.Application;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

import java.text.DecimalFormat;

public class MyApplication extends Application {
    private static DecimalFormat VALUE_FORMAT = new DecimalFormat("000.00");

    private Node createKnobBox(double initialValue) {
        Knob knob = new Knob(initialValue);
        
        Label label = new Label(VALUE_FORMAT.format(initialValue));
        VBox box = new VBox(knob, label);
        box.setStyle("-fx-alignment: center; -fx-spacing: 5");

        knob.valueProperty().addListener(new ChangeListener<Number>() {
            @Override
            public void changed(ObservableValue<? extends Number> observable,
                                Number oldValue, Number newValue) {
                label.setText(VALUE_FORMAT.format(newValue));
            }
        });

        return box;
    }

    @Override
    public void start(Stage primaryStage) throws Exception {
        String css = getClass().getResource("knob.css").toExternalForm();
        StyleManager.getInstance().addUserAgentStylesheet(css);
        
        HBox rootPane = new HBox(createKnobBox(0), createKnobBox(50), createKnobBox(100));
        rootPane.setAlignment(Pos.CENTER);
        rootPane.setStyle("-fx-padding: 10");

        primaryStage.setScene(new Scene(rootPane, 300, 200));
        primaryStage.setTitle("Styleable Knob");
        primaryStage.show();
    }
}
